﻿if not HCoold then return false end
local AceConfig = LibStub("AceConfigDialog-3.0")
local L = HCoold:GetLocale()

local function t01to16(num)
	local tmp = num * 255
	local a1 = math.floor(tmp/16)
	local a2 = math.floor(tmp - a1*16)
	local out = ""

	if a1 < 10 then out = tostring(a1) end
	if a1 == 10 then out = "A" end
	if a1 == 11 then out = "B" end
	if a1 == 12 then out = "C" end
	if a1 == 13 then out = "D" end
	if a1 == 14 then out = "E" end
	if a1 == 15 then out = "F" end

	if a2 < 10 then out = out .. tostring(a2) end
	if a2 == 10 then out = out .. "A" end
	if a2 == 11 then out = out .. "B" end
	if a2 == 12 then out = out .. "C" end
	if a2 == 13 then out = out .. "D" end
	if a2 == 14 then out = out .. "E" end
	if a2 == 15 then out = out .. "F" end
	
	return out
end

local function t16to01(str)
	local a1 = string.sub(str,1,1)
	local a2 = string.sub(str,2)
	local s1 = tonumber(a1)
	local s2 = tonumber(a2)
	
	if string.lower(a1) == "a" then s1 = 10 end
	if string.lower(a1) == "b" then s1 = 11 end
	if string.lower(a1) == "c" then s1 = 12 end
	if string.lower(a1) == "d" then s1 = 13 end
	if string.lower(a1) == "e" then s1 = 14 end
	if string.lower(a1) == "f" then s1 = 15 end

	if string.lower(a2) == "a" then s2 = 10 end
	if string.lower(a2) == "b" then s2 = 11 end
	if string.lower(a2) == "c" then s2 = 12 end
	if string.lower(a2) == "d" then s2 = 13 end
	if string.lower(a2) == "e" then s2 = 14 end
	if string.lower(a2) == "f" then s2 = 15 end
	
	local out = (s1*16+s2)/255
	
	return out
end

local function getRGBA(color)
	local a = string.sub(color,3,4)
	local r = string.sub(color,5,6)
	local g = string.sub(color,7,8)
	local b = string.sub(color,9,10)
	
	local a_ = t16to01(a)
	local r_ = t16to01(r)
	local g_ = t16to01(g)
	local b_ = t16to01(b)
	
	return r_, g_, b_, a_
end

local function setRGBA(r,g,b,a)
	local out = string.format("|c%s%s%s%s",t01to16(a),t01to16(r),t01to16(g),t01to16(b))
	return out
end

function HCoold:GenerateOptions()
	local options = {
		type = "group",
		get = getProfileOption,
		set = setProfileOptionAndClearCache,
		args = {
			lock = {
				type = "execute",
				name = L["lock frames"],
				desc = L["desc lock frames"],
				order = 1,
				func = function () self:LockFrames() end,
			},
			unlock = {
				type = "execute",
				name = L["unlock frames"],
				desc = L["desc unlock frames"],
				order = 2,
				func = function () self:UnlockFrames() end,
			},
			wipe = {
				type = "execute",
				name = L["redraw"],
				desc = L["desc redraw"],
				cmdHidden = true,
				order = 3,
				confirm = true,
				func = function () self:WIPE() end,
			},
			config = {
				type = "execute",
				guiHidden = true,
				name = L["run config"],
				desc = L["desc run config"],
				order = 4,
				func = function ()
					AceConfig:SetDefaultSize(700,700)
					AceConfig:Open(self.name)
				end,
			},
			hide = {
				type = "execute",
				name = L["show/hide all"],
				desc = L["desc show/hide all"],
				order = 5,
				func = function ()
					local show = false
					for _, i in next, self.types do if i:Enable() then show = true end end
					for _, i in next, self.types do 
						i:Enable(not show)
						self.db.profile.types[i.type].enable = not show
					end
				end,
			},
			spells = {
				type = "group",
				cmdHidden = true,
				name = L["spells"],
				desc = L["desc spells"],
				order = 5,
				args = HCoold:GenerateSpellList()
			},
		},
	}
	
	options.args.spells.args.header = {
		type = "header",
		order = 3,
		name = L["color setup"],
	}
	do -- font config
		local LSM = LibStub("LibSharedMedia-3.0")
		options.args.spells.args.font = {
			type = "select",
			order = 1,
			name = L["font name"],
			desc = L["desc font name"],
			values = LSM:HashTable("font"),
			dialogControl = "LSM30_Font",
			set = function(_,val) self.db.profile.font.name = val end,
			get = function() return self.db.profile.font.name end,
			cmdHidden = true,
		}
		options.args.spells.args.font_size = {
			type = "input",
			order = 2,
			width = "half",
			name = L["font size"],
			desc = L["desc font size"],
			set = function(_,val) 
				local tmp = tonumber(val) or 18
				if tmp <= 0 then tmp = 18 end
				self.db.profile.font.size = tmp
			end,
			get = function() return tostring(self.db.profile.font.size) end,
			cmdHidden = true,
		}
	end
	do -- color config
		options.args.spells.args.exl_color = {
			type = "color",
			order = 4,
			name = L["exl spell color"],
			desc = L["desc exl spell color"],
			hasAlpha = true,
			set = function(_,r,g,b,a) self.db.profile.color.supergood = setRGBA(r,g,b,a) end,
			get = function() return getRGBA(self.db.profile.color.supergood) end,
			cmdHidden = true,
		}
		options.args.spells.args.good_color = {
			type = "color",
			order = 5,
			name = L["good spell color"],
			hasAlpha = true,
			desc = L["desc good spell color"],
			set = function() self.db.profile.color.good = setRGBA(r,g,b,a) end,
			get = function() return getRGBA(self.db.profile.color.good) end,
			cmdHidden = true,
		}
		options.args.spells.args.bad_color = {
			type = "color",
			order = 6,
			name = L["bad spell color"],
			hasAlpha = true,
			desc = L["desc bad spell color"],
			set = function() self.db.profile.color.bad = setRGBA(r,g,b,a) end,
			get = function() return getRGBA(self.db.profile.color.bad) end,
			cmdHidden = true,
		}
		options.args.spells.args.off_color = {
			type = "color",
			order = 7,
			name = L["off spell color"],
			hasAlpha = true,
			desc = L["desc off spell color"],
			set = function() self.db.profile.color.offline = setRGBA(r,g,b,a) end,
			get = function() return getRGBA(self.db.profile.color.offline) end,
			cmdHidden = true,
		}
		options.args.spells.args.dead_color = {
			type = "color",
			order = 8,
			name = L["dead spell color"],
			desc = L["desc dead spell color"],
			hasAlpha = true,
			set = function() self.db.profile.color.dead = setRGBA(r,g,b,a) end,
			get = function() return getRGBA(self.db.profile.color.dead) end,
			cmdHidden = true,
		}
		options.args.spells.args.cd_color = {
			type = "color",
			order = 9,
			name = L["cd spell color"],
			desc = L["desc cd spell color"],
			hasAlpha = true,
			set = function() self.db.profile.color.cd = setRGBA(r,g,b,a) end,
			get = function() return getRGBA(self.db.profile.color.cd) end,
			cmdHidden = true,
		}
	end
	
	options.args.profiles = LibStub('AceDBOptions-3.0'):GetOptionsTable(HCoold.db)
	options.args.profiles.order = -1
	options.args.profiles.cmdHidden = true
	options.args.profiles.disabled = false
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable(self.name, options, "hcd")
	AceConfig:AddToBlizOptions(self.name,L["Hagakure cooldowns"])
end






































